"""Command line for Buildout Versions Checker"""
from six import string_types

import sys
import copy
import logging
from argparse import Action
from argparse import ArgumentError
from argparse import ArgumentParser
from argparse import _ensure_value

from bvc.logger import logger
from bvc.checker import VersionsChecker
from bvc.configparser import VersionsConfigParser


class StoreSpecifiers(Action):

    def __call__(self, parser, namespace, values, option_string=None):
        items = copy.copy(_ensure_value(namespace, self.dest, {}))
        try:
            key, value = values.split(':')
        except ValueError:
            raise ArgumentError(self, 'key:value syntax not followed')
        key = key.strip()
        value = value.strip()
        if not key or not value:
            raise ArgumentError(self, 'key or value are empty')
        items.update({key: value})
        setattr(namespace, self.dest, items)


def cmdline(argv=sys.argv[1:]):
    parser = ArgumentParser(
        description='Check availables updates from a '
        'version section of a buildout script')
    parser.add_argument(
        'source', default='versions.cfg', nargs='?',
        help='The file where versions are pinned '
        '(default: versions.cfg)')
    version_group = parser.add_argument_group('Allowed versions')
    version_group.add_argument(
        '--pre', action='store_true', dest='prereleases', default=False,
        help='Allow pre-releases and development versions '
        '(by default only stable versions are found)')
    version_group.add_argument(
        '-s', '--specifier', action=StoreSpecifiers,
        dest='specifiers', default={},
        help='Describe what versions of a package are acceptable. '
        'Example "package:>=1.0,!=1.3.4.*,< 2.0" '
        '(can be used multiple times)')
    filter_group = parser.add_argument_group('Filtering')
    filter_group.add_argument(
        '-i', '--include', action='append', dest='includes', default=[],
        help='Include package when checking updates '
        '(can be used multiple times)')
    filter_group.add_argument(
        '-e', '--exclude', action='append', dest='excludes', default=[],
        help='Exclude package when checking updates '
        '(can be used multiple times)')
    file_group = parser.add_argument_group('File')
    file_group.add_argument(
        '-w', '--write', action='store_true', dest='write', default=False,
        help='Write the updates in the source file')
    file_group.add_argument(
        '--indent', dest='indentation', type=int, default=32,
        help='Spaces used when indenting "key = value" (default: 32)')
    file_group.add_argument(
        '--sorting', dest='sorting', default='', choices=['alpha', 'length'],
        help='Sorting algorithm used on the keys when writing source file '
        '(default: None)')
    network_group = parser.add_argument_group('Network')
    network_group.add_argument(
        '--service-url',  dest='service_url',
        default='http://pypi.python.org/pypi',
        help='The service to use for checking the packages '
        '(default: http://pypi.python.org/pypi)')
    network_group.add_argument(
        '--timeout', dest='timeout', type=int, default=10,
        help='Timeout for each request (default: 10s)')
    network_group.add_argument(
        '-t', '--threads', dest='threads', type=int, default=10,
        help='Threads used for checking the versions in parallel')
    verbosity_group = parser.add_argument_group('Verbosity')
    verbosity_group.add_argument(
        '-v', action='count', dest='verbosity', default=1,
        help='Increase verbosity (specify multiple times for more)')
    verbosity_group.add_argument(
        '-q', action='count', dest='quietly', default=0,
        help='Decrease verbosity (specify multiple times for more)')

    if isinstance(argv, string_types):
        argv = argv.split()
    options = parser.parse_args(argv)

    verbose_logs = {0: 100,
                    1: logging.WARNING,
                    2: logging.INFO,
                    3: logging.DEBUG}
    verbosity = min(3, max(0, options.verbosity - options.quietly))
    console = logging.StreamHandler(sys.stdout)
    console.setLevel(verbose_logs[verbosity])
    logger.addHandler(console)

    source = options.source
    try:
        checker = VersionsChecker(
            source,
            options.specifiers, options.prereleases,
            options.includes, options.excludes,
            options.service_url, options.timeout,
            options.threads)
    except Exception as e:
        sys.exit(str(e))

    if not checker.updates:
        sys.exit(0)

    logger.warning('[versions]')
    for package, version in checker.updates.items():
        logger.warning('%s= %s' % (
            package.ljust(options.indentation), version))

    if options.write:
        config = VersionsConfigParser()
        config.read(source)
        if not config.has_section('versions'):
            config.add_section('versions')
        for package, version in checker.updates.items():
            config.set('versions', package, version)

        config.write(source, options.indentation, options.sorting)
        logger.info('- %s updated.' % source)

    sys.exit(0)



"""Command line for finding unused pinned versions"""
from six import string_types

import sys
import logging
from argparse import ArgumentParser

from bvc.logger import logger
from bvc.checker import UnusedVersionsChecker
from bvc.configparser import VersionsConfigParser


def cmdline(argv=sys.argv[1:]):
    parser = ArgumentParser(
        description='Find unused pinned eggs')
    parser.add_argument(
        'source', default='versions.cfg', nargs='?',
        help='The file where versions are pinned '
        '(default: versions.cfg)')
    filter_group = parser.add_argument_group('Filtering')
    filter_group.add_argument(
        '--eggs', dest='eggs', default='./eggs/',
        help='The directory where the eggs are located '
        '(default: ./eggs/)')
    filter_group.add_argument(
        '-e', '--exclude', action='append', dest='excludes', default=[],
        help='Exclude package when checking updates '
        '(can be used multiple times)')
    file_group = parser.add_argument_group('File')
    file_group.add_argument(
        '-w', '--write', action='store_true', dest='write', default=False,
        help='Write the updates in the source file')
    file_group.add_argument(
        '--indent', dest='indentation', type=int, default=32,
        help='Spaces used when indenting "key = value" (default: 32)')
    file_group.add_argument(
        '--sorting', dest='sorting', default='', choices=['alpha', 'length'],
        help='Sorting algorithm used on the keys when writing source file '
        '(default: None)')
    verbosity_group = parser.add_argument_group('Verbosity')
    verbosity_group.add_argument(
        '-v', action='count', dest='verbosity', default=1,
        help='Increase verbosity (specify multiple times for more)')
    verbosity_group.add_argument(
        '-q', action='count', dest='quietly', default=0,
        help='Decrease verbosity (specify multiple times for more)')

    if isinstance(argv, string_types):
        argv = argv.split()
    options = parser.parse_args(argv)

    verbose_logs = {0: 100,
                    1: logging.WARNING,
                    2: logging.INFO,
                    3: logging.DEBUG}
    verbosity = min(3, max(0, options.verbosity - options.quietly))
    console = logging.StreamHandler(sys.stdout)
    console.setLevel(verbose_logs[verbosity])
    logger.addHandler(console)

    source = options.source
    try:
        checker = UnusedVersionsChecker(
            source, options.eggs, options.excludes)
    except Exception as e:
        sys.exit(str(e))

    if not checker.unused:
        sys.exit(0)

    for package in checker.unused:
        logger.warning('- %s is unused.' % package)

    if options.write:
        config = VersionsConfigParser()
        config.read(source)
        for package in checker.unused:
            config.remove_option('versions', package)

        config.write(source, options.indentation, options.sorting)
        logger.info('- %s updated.' % source)

    sys.exit(0)



"""Config parser for Buildout Versions Checker"""
import re
from operator import itemgetter
try:
    from ConfigParser import RawConfigParser
except ImportError:  # Python 3
    from configparser import RawConfigParser

OPERATORS = re.compile(r'[+-]$')


class VersionsConfigParser(RawConfigParser):
    """
    ConfigParser customized to read and write
    beautiful buildout files.
    """
    optionxform = str

    def alpha_sorter(self, items):
        return sorted(items, key=itemgetter(0))

    def length_sorter(self, items):
        return sorted(self.alpha_sorter(items),
                      key=lambda x: len(x[0]))

    def write_section(self, fd, section, indentation, sorting):
        """
        Write a section of an .ini-format
        and all the keys within.
        """
        string_section = '[%s]\n' % section

        items = self._sections[section].items()
        try:
            items = getattr(self, '%s_sorter' % sorting)(items)
        except (TypeError, AttributeError):
            pass

        for key, value in items:
            if key == '__name__':
                continue
            if value is None:
                value = ''
            operator = ''
            buildout_operator = OPERATORS.search(key)
            if buildout_operator:
                operator = buildout_operator.group(0)
                key = key[:-1]
            if key == '<':
                value = '{value:>{indent}}'.format(
                    value=value, indent=indentation + len(value) - 1)
            else:
                key = '{key:<{indent}}{operator}'.format(
                    key=key, operator=operator,
                    indent=indentation - int(bool(operator)))
            value = value.replace('\n', '{:<{indent}}'.format(
                '\n', indent=indentation + 3))
            string_section += '{key}= {value}\n'.format(key=key, value=value)

        fd.write(string_section.encode('utf-8'))

    def write(self, source, indentation=32, sorting=None):
        """
        Write an .ini-format representation of the
        configuration state with a readable indentation.
        """
        with open(source, 'wb') as fd:
            sections = list(self._sections.keys())
            for section in sections[:-1]:
                self.write_section(fd, section, indentation, sorting)
                fd.write('\n'.encode('utf-8'))
            self.write_section(fd, sections[-1], indentation, sorting)



"""Buildout Versions Checker"""
from bvc.checker import VersionsChecker
from bvc.configparser import VersionsConfigParser
from bvc.scripts.check_buildout_updates import cmdline

__all__ = [
    VersionsChecker.__name__,
    VersionsConfigParser.__name__
]

if __name__ == '__main__':
    cmdline()  # pragma: no cover



"""Tests for Buildout version checker"""
import os
import sys
import json

from logging import Handler
from collections import OrderedDict
from tempfile import NamedTemporaryFile
try:
    from urllib2 import URLError
    from cStringIO import StringIO
except ImportError:  # Python 3
    from io import StringIO
    from urllib.error import URLError

from unittest import TestCase
from unittest import TestSuite
from unittest import TestLoader

from bvc import checker
from bvc.logger import logger
from bvc.checker import VersionsChecker
from bvc.checker import UnusedVersionsChecker
from bvc.scripts import indent_buildout
from bvc.scripts import find_unused_versions
from bvc.scripts import check_buildout_updates
from bvc.configparser import VersionsConfigParser


class LazyVersionsChecker(VersionsChecker):
    """
    VersionsChecker who does nothing at the initialisation
    excepting recording the arguments.
    """

    def __init__(self, **kw):
        for key, value in kw.items():
            setattr(self, key, value)


class LazyUnusedVersionsChecker(UnusedVersionsChecker):
    """
    UnusedVersionsChecker who does nothing at the
    initialisation excepting recording the arguments.
    """

    def __init__(self, **kw):
        for key, value in kw.items():
            setattr(self, key, value)


class URLOpener(object):
    """
    Fake urlopen.
    """
    results = {
        'egg': {
            'releases': ['0.3', '0.2']
        },
        'egg-dev': {
            'releases': ['1.0', '1.1b1']
        },
        'error-egg': [],
    }

    def __call__(self, url):
        package = url.split('/')[-2]
        try:
            return StringIO(json.dumps(self.results[package]))
        except KeyError:
            raise URLError('404')


class StubbedURLOpenTestCase(TestCase):
    """
    TestCase enabling a stub around the urllib2.urlopen
    used by VersionsChecker.
    """

    def setUp(self):
        self.stub_url_open()
        super(StubbedURLOpenTestCase, self).setUp()

    def tearDown(self):
        self.unstub_url_open()
        super(StubbedURLOpenTestCase, self).tearDown()

    def stub_url_open(self):
        """
        Replace the urlopen used in bvc.
        """
        self.original_url_open = checker.urlopen
        checker.urlopen = URLOpener()

    def unstub_url_open(self):
        """
        Restaure the original urlopen function.
        """
        checker.urlopen = self.original_url_open


class StubbedListDirTestCase(TestCase):
    """
    TestCase for faking the os.listdir calls.
    """
    listdir_content = []

    def setUp(self):
        self.stub_listdir()
        super(StubbedListDirTestCase, self).setUp()

    def tearDown(self):
        self.unstub_listdir()
        super(StubbedListDirTestCase, self).tearDown()

    def stub_listdir(self):
        """
        Replace the os.listdir function.
        """
        self.original_listdir = os.listdir
        os.listdir = lambda x: self.listdir_content

    def unstub_listdir(self):
        """
        Restaure the original os.listdir function.
        """
        os.listdir = self.original_listdir


class DictHandler(Handler):
    """
    Logging handler to check for expected logs.
    """

    def __init__(self, *ka, **kw):
        self.messages = {
            'debug': [], 'info': [],
            'warning': [], 'error': [],
            'critical': []
        }
        super(DictHandler, self).__init__(*ka, **kw)

    def emit(self, record):
        self.messages[record.levelname.lower()
                      ].append(record.getMessage())


class LogsTestCase(TestCase):
    """
    TestCase allowing to check the messages
    emitted by the logs.
    """

    def setUp(self):
        self.logs = DictHandler()
        logger.addHandler(self.logs)
        super(LogsTestCase, self).setUp()

    def tearDown(self):
        logger.removeHandler(self.logs)
        super(LogsTestCase, self).tearDown()

    def assertLogs(self, debug=[], info=[], warning=[],
                   error=[], critical=[]):
        expected = {'debug': debug, 'info': info,
                    'warning': warning, 'error': error,
                    'critical': critical}
        for key, item in expected.items():
            self.assertEquals(self.logs.messages[key],
                              item)


class StdOutTestCase(TestCase):
    """
    TestCase for capturing printed output on stdout.
    """

    def setUp(self):
        self.output = StringIO()
        self.saved_stdout = sys.stdout
        self.saved_stderr = sys.stderr
        sys.stdout = self.output
        sys.stderr = self.output
        super(StdOutTestCase, self).setUp()

    def tearDown(self):
        sys.stdout = self.saved_stdout
        sys.stderr = self.saved_stderr
        super(StdOutTestCase, self).tearDown()

    def assertStdOut(self, output):
        self.assertEquals(self.output.getvalue(),
                          output)

    def assertInStdOut(self, output):
        self.assertTrue(output in self.output.getvalue())


class VersionsCheckerTestCase(StubbedURLOpenTestCase):

    def setUp(self):
        self.checker = LazyVersionsChecker(
            service_url='http://custom.pypi.org/pypi')
        super(VersionsCheckerTestCase, self).setUp()

    def test_parse_versions(self):
        config_file = NamedTemporaryFile()
        config_file.write('[sections]\nKey=Value\n'.encode('utf-8'))
        config_file.seek(0)
        self.assertEquals(self.checker.parse_versions(config_file.name),
                          [])
        config_file.seek(0)
        config_file.write('[VERSIONS]\negg=0.1\nEgg = 0.2'.encode('utf-8'))
        config_file.seek(0)
        self.assertEquals(self.checker.parse_versions(config_file.name),
                          [])
        config_file.seek(0)
        config_file.write('[versions]\negg=0.1\nEgg = 0.2'.encode('utf-8'))
        config_file.seek(0)
        self.assertEquals(self.checker.parse_versions(config_file.name),
                          [('egg', '0.1'), ('Egg', '0.2')])
        config_file.close()

    def test_include_exclude_versions(self):
        source_versions = OrderedDict([('egg', '0.1'), ('Egg', '0.2')])
        self.assertEquals(self.checker.include_exclude_versions(
            source_versions), source_versions)
        results = source_versions.copy()
        results['Django'] = '0.0.0'
        self.assertEquals(
            self.checker.include_exclude_versions(
                source_versions, includes=['Django', 'egg']),
            results)
        source_versions['Django'] = '1.5.1'
        source_versions['pytz'] = '2013b'
        results = OrderedDict([('pytz', '2013b')])
        self.assertEquals(
            self.checker.include_exclude_versions(
                source_versions, excludes=['Django', 'egg']),
            results)
        self.assertEquals(
            self.checker.include_exclude_versions(
                source_versions,
                includes=['Django', 'egg'],
                excludes=['Django', 'egg']),
            results)
        results['zc.buildout'] = '0.0.0'
        self.assertEquals(
            self.checker.include_exclude_versions(
                source_versions,
                includes=['zc.buildout'],
                excludes=['Django', 'egg']),
            results)

    def test_build_specifiers(self):
        self.assertEquals(
            self.checker.build_specifiers(
                ('Django', 'zc.buildout'),
                {'django': '<=1.8',
                 'extra': '!=1.2'}),
            [('Django', '<=1.8'), ('zc.buildout', '')])

    def test_fetch_last_versions(self):
        self.assertEquals(
            self.checker.fetch_last_versions(
                [('egg', ''), ('UnknowEgg', '')], False,
                'service_url', 1, 1),
            [('egg', '0.3'), ('UnknowEgg', '0.0.0')])
        self.assertEquals(
            self.checker.fetch_last_versions(
                [('egg', '<=0.2'), ('UnknowEgg', '>1.0')], False,
                'service_url', 1, 1),
            [('egg', '0.2'), ('UnknowEgg', '0.0.0')])
        results = self.checker.fetch_last_versions(
            [('egg', ''), ('UnknowEgg', '')], False,
            'service_url', 1, 2)
        self.assertEquals(
            dict(results),
            dict([('egg', '0.3'), ('UnknowEgg', '0.0.0')]))

    def test_fetch_last_version(self):
        self.assertEquals(
            self.checker.fetch_last_version(
                ('UnknowEgg', ''), False, 'service_url', 1),
            ('UnknowEgg', '0.0.0')
        )
        self.assertEquals(
            self.checker.fetch_last_version(
                ('egg', ''), False, 'service_url', 1),
            ('egg', '0.3')
        )
        self.assertEquals(
            self.checker.fetch_last_version(
                ('egg', '<0.3'), False, 'service_url', 1),
            ('egg', '0.2')
        )

    def test_fetch_last_version_with_prereleases(self):
        self.assertEquals(
            self.checker.fetch_last_version(
                ('egg-dev', ''), False, 'service_url', 1),
            ('egg-dev', '1.0')
        )
        self.assertEquals(
            self.checker.fetch_last_version(
                ('egg-dev', ''), True, 'service_url', 1),
            ('egg-dev', '1.1b1')
        )
        self.assertEquals(
            self.checker.fetch_last_version(
                ('egg-dev', '<1.1'), True, 'service_url', 1),
            ('egg-dev', '1.0')
        )
        self.assertEquals(
            self.checker.fetch_last_version(
                ('egg-dev', '<=1.1'), True, 'service_url', 1),
            ('egg-dev', '1.1b1')
        )

    def test_find_updates(self):
        versions = OrderedDict([('egg', '1.5.1'), ('Egg', '0.0.0')])
        last_versions = OrderedDict([('egg', '1.5.1'), ('Egg', '1.0')])
        self.assertEquals(self.checker.find_updates(
            versions, last_versions), [('Egg', '1.0')])


class UnusedVersionsCheckerTestCase(StubbedListDirTestCase):

    def setUp(self):
        self.checker = LazyUnusedVersionsChecker()
        super(UnusedVersionsCheckerTestCase, self).setUp()

    def test_get_used_versions(self):
        self.listdir_content = ['file',
                                'package-1.0.egg',
                                'composed_egg-1.0.egg']
        self.assertEquals(self.checker.get_used_versions('.'),
                          ['package', 'composed_egg'])

    def test_get_find_unused_versions(self):
        self.assertEquals(
            self.checker.find_unused_versions(
                ['egg', 'CAPegg', 'composed-egg', 'unused'],
                ['Egg', 'capegg', 'composed_egg']),
            ['unused'])


class VersionsConfigParserTestCase(TestCase):

    def test_parse_case_insensitive(self):
        config_file = NamedTemporaryFile()
        config_file.write('[Section]\nKEY=VALUE\nKey=Value\n'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        self.assertEquals(config_parser.sections(), ['Section'])
        self.assertEquals(config_parser.options('Section'), ['KEY', 'Key'])
        config_file.close()

    def test_write_section(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section')
        config_parser.set('Section', 'Option', 'Value')
        config_parser.set('Section', 'Option-void', None)
        config_parser.set('Section', 'Option-multiline', 'Value1\nValue2')
        config_parser.write_section(config_file, 'Section', 24, '')
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Option                  = Value\n'
            'Option-void             = \n'
            'Option-multiline        = Value1\n'
            '                          Value2\n')
        config_file.close()

    def test_write_section_alpha_sorting(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section')
        config_parser.set('Section', 'Option-multiline', 'Value1\nValue2')
        config_parser.set('Section', 'Option-void', None)
        config_parser.set('Section', 'Option', 'Value')
        config_parser.write_section(config_file, 'Section', 24, 'alpha')
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Option                  = Value\n'
            'Option-multiline        = Value1\n'
            '                          Value2\n'
            'Option-void             = \n')
        config_file.close()

    def test_write_section_length_sorting(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section')
        config_parser.set('Section', 'Option-multiline', 'Value1\nValue2')
        config_parser.set('Section', 'Option-void', None)
        config_parser.set('Section', 'Option-size', None)
        config_parser.set('Section', 'Option', 'Value')
        config_parser.write_section(config_file, 'Section', 24, 'length')
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Option                  = Value\n'
            'Option-size             = \n'
            'Option-void             = \n'
            'Option-multiline        = Value1\n'
            '                          Value2\n')
        config_file.close()

    def test_write_section_low_indentation(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section')
        config_parser.set('Section', 'Option', 'Value')
        config_parser.set('Section', 'Option-void', None)
        config_parser.set('Section', 'Option-multiline', 'Value1\nValue2')
        config_parser.write_section(config_file, 'Section', 12, '')
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Option      = Value\n'
            'Option-void = \n'
            'Option-multiline= Value1\n'
            '              Value2\n')
        config_file.close()

    def test_write(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section 1')
        config_parser.add_section('Section 2')
        config_parser.set('Section 1', 'Option', 'Value')
        config_parser.set('Section 1', 'Option-void', None)
        config_parser.set('Section 1', 'Option-add+', 'Value added')
        config_parser.set('Section 2', 'Option-multiline', 'Value1\nValue2')
        config_parser.set('Section 2', '<', 'Value1\nValue2')
        config_parser.write(config_file.name)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section 1]\n'
            'Option                          = Value\n'
            'Option-void                     = \n'
            'Option-add                     += Value added\n'
            '\n'
            '[Section 2]\n'
            'Option-multiline                = Value1\n'
            '                                  Value2\n'
            '<=                                Value1\n'
            '                                  Value2\n')
        config_file.close()

    def test_write_alpha_sorting(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section 1')
        config_parser.add_section('Section 2')
        config_parser.set('Section 1', 'Option', 'Value')
        config_parser.set('Section 1', 'Option-void', None)
        config_parser.set('Section 1', 'Option-add+', 'Value added')
        config_parser.set('Section 2', 'Option-multiline', 'Value1\nValue2')
        config_parser.set('Section 2', '<', 'Value1\nValue2')
        config_parser.write(config_file.name, sorting='alpha')
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section 1]\n'
            'Option                          = Value\n'
            'Option-add                     += Value added\n'
            'Option-void                     = \n'
            '\n'
            '[Section 2]\n'
            '<=                                Value1\n'
            '                                  Value2\n'
            'Option-multiline                = Value1\n'
            '                                  Value2\n')
        config_file.close()

    def test_write_low_indentation(self):
        config_file = NamedTemporaryFile()
        config_parser = VersionsConfigParser()
        config_parser.add_section('Section 1')
        config_parser.add_section('Section 2')
        config_parser.set('Section 1', 'Option', 'Value')
        config_parser.set('Section 1', 'Option-void', None)
        config_parser.set('Section 2', 'Option-multiline', 'Value1\nValue2')
        config_parser.write(config_file.name, 12)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section 1]\n'
            'Option      = Value\n'
            'Option-void = \n'
            '\n'
            '[Section 2]\n'
            'Option-multiline= Value1\n'
            '              Value2\n')
        config_file.close()

    def test_parse_and_write_buildout_operators(self):
        config_file = NamedTemporaryFile()
        config_file.write(
            '[Section]\nAd+=dition\nSub-=straction'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        config_file.seek(0)
        config_parser.write(config_file.name, 24)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Ad                     += dition\n'
            'Sub                    -= straction\n')
        config_file.close()

    def test_parse_and_write_buildout_operators_offset(self):
        config_file = NamedTemporaryFile()
        config_file.write(
            '[Section]\nAd  +=dition\nSub - = straction'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        config_file.seek(0)
        config_parser.write(config_file.name, 24)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Ad                     += dition\n'
            'Sub                    -= straction\n')
        config_file.close()

    def test_parse_and_write_buildout_operators_multilines(self):
        config_file = NamedTemporaryFile()
        config_file.write(
            '[Section]\nAdd+=Multi\n  Lines'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        config_file.seek(0)
        config_parser.write(config_file.name, 24)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            'Add                    += Multi\n'
            '                          Lines\n')
        config_file.close()

    def test_parse_and_write_buildout_macros(self):
        config_file = NamedTemporaryFile()
        config_file.write(
            '[Section]\n<=Macro\n Template'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        config_file.seek(0)
        config_parser.write(config_file.name, 24)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            '<=                        Macro\n'
            '                          Template\n')
        config_file.close()

    def test_parse_and_write_buildout_macros_offset(self):
        config_file = NamedTemporaryFile()
        config_file.write(
            '[Section]\n<  = Macro\n  Template'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        config_file.seek(0)
        config_parser.write(config_file.name, 24)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section]\n'
            '<=                        Macro\n'
            '                          Template\n')
        config_file.close()

    def test_parse_and_write_buildout_conditional_sections(self):
        config_file = NamedTemporaryFile()
        config_file.write('[Section:Condition]\nKey=Value\n'.encode('utf-8'))
        config_file.seek(0)
        config_parser = VersionsConfigParser()
        config_parser.read(config_file.name)
        config_file.seek(0)
        config_parser.write(config_file.name, 24)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[Section:Condition]\n'
            'Key                     = Value\n')
        config_file.close()


class FindUnusedVersionsTestCase(LogsTestCase,
                                 StdOutTestCase,
                                 StubbedListDirTestCase):
    listdir_content = [
        'egg-1.0.egg',
        'composed_egg-1.0.egg']

    def test_simple(self):
        config_file = NamedTemporaryFile()
        config_file.write('[versions]\nEgg=1.0\n'
                          'Unused-egg=1.0\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline(config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            info=['- 2 versions found in %s.' % config_file.name,
                  '- 2 packages need to be checked for updates.'],
            warning=['- Unused-egg is unused.'])
        self.assertStdOut('- Unused-egg is unused.\n')
        config_file.close()

    def test_write(self):
        config_file = NamedTemporaryFile()
        config_file.write('[versions]\nEgg=1.0\n'
                          'Unused-egg=1.0\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('%s -w' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            info=['- 2 versions found in %s.' % config_file.name,
                  '- 2 packages need to be checked for updates.',
                  '- %s updated.' % config_file.name],
            warning=['- Unused-egg is unused.'])
        self.assertStdOut('- Unused-egg is unused.\n')
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[versions]\nEgg                             = 1.0\n')

    def test_no_source(self):
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            debug=["'versions' section not found in versions.cfg."],
            info=['- 0 packages need to be checked for updates.'])
        self.assertStdOut('')

    def test_exclude(self):
        config_file = NamedTemporaryFile()
        config_file.write('[versions]\nEgg=1.0\n'
                          'Unused-egg=1.0\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('%s -e unused-egg' %
                                         config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            info=['- 2 versions found in %s.' % config_file.name,
                  '- 1 packages need to be checked for updates.'])
        self.assertStdOut('')
        config_file.close()

    def test_output_max(self):
        config_file = NamedTemporaryFile()
        config_file.write('[versions]\nEgg=1.0\n'
                          'Unused-egg=1.0\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('%s -vvv' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            info=['- 2 versions found in %s.' % config_file.name,
                  '- 2 packages need to be checked for updates.'],
            warning=['- Unused-egg is unused.'])
        self.assertStdOut(
            '- 2 versions found in %s.\n'
            '- 2 packages need to be checked for updates.\n'
            '- Unused-egg is unused.\n' % config_file.name)
        config_file.close()

    def test_output_none(self):
        config_file = NamedTemporaryFile()
        config_file.write('[versions]\nEgg=1.0\n'
                          'Unused-egg=1.0\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('%s -q' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut('')
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('invalid -qqqq')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut('')
        config_file.close()

    def test_handle_error(self):
        original_listdir_content = self.listdir_content
        self.listdir_content = 42
        config_file = NamedTemporaryFile()
        config_file.write('[versions]\n'.encode('utf-8'))
        with self.assertRaises(SystemExit) as context:
            find_unused_versions.cmdline('%s' % config_file.name)
        self.assertEqual(context.exception.code,
                         "'int' object is not iterable")
        self.listdir_content = original_listdir_content


class IndentCommandLineTestCase(LogsTestCase,
                                StdOutTestCase):

    def test_simple(self):
        config_file = NamedTemporaryFile()
        config_file.write('[sections]\nKey=Value\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            indent_buildout.cmdline('%s' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            warning=['- %s (re)indented at 32 spaces.' % config_file.name])
        self.assertStdOut(
            '- %s (re)indented at 32 spaces.\n' % config_file.name)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[sections]\n'
            'Key                             = Value\n')
        config_file.close()

    def test_invalid_source(self):
        with self.assertRaises(SystemExit) as context:
            indent_buildout.cmdline('invalid.cfg')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(warning=['- invalid.cfg cannot be read.'])
        self.assertStdOut('- invalid.cfg cannot be read.\n')

    def test_multiple_sources(self):
        config_file = NamedTemporaryFile()
        config_file.write('[sections]\nKey=Value\n'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            indent_buildout.cmdline('%s invalid.cfg' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            warning=['- %s (re)indented at 32 spaces.' % config_file.name,
                     '- invalid.cfg cannot be read.'])
        self.assertStdOut(
            '- %s (re)indented at 32 spaces.\n'
            '- invalid.cfg cannot be read.\n' % config_file.name)

    def test_no_source(self):
        with self.assertRaises(SystemExit) as context:
            indent_buildout.cmdline('')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            warning=['No files to (re)indent'])
        self.assertStdOut(
            'No files to (re)indent\n')

    def test_output_none(self):
        with self.assertRaises(SystemExit) as context:
            indent_buildout.cmdline('invalid.cfg -q')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut('')
        with self.assertRaises(SystemExit) as context:
            indent_buildout.cmdline('source.cfg -qqqqqqq')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut('')


class CheckUpdatesCommandLineTestCase(LogsTestCase,
                                      StdOutTestCase,
                                      StubbedURLOpenTestCase):

    def test_no_args_no_source(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            debug=["'versions' section not found in versions.cfg."],
            info=['- 0 packages need to be checked for updates.',
                  '- 0 package updates found.'])
        self.assertStdOut('')

    def test_include_no_source(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            debug=["'versions' section not found in versions.cfg.",
                   '-> Last version of egg is 0.3.',
                   '=> egg current version (0.0.0) and '
                   'last version (0.3) are different.'],
            info=['- 1 packages need to be checked for updates.',
                  '> Fetching latest datas for egg...',
                  '- 1 package updates found.'],
            warning=['[versions]',
                     'egg                             = 0.3'])
        self.assertStdOut(
            '[versions]\n'
            'egg                             = 0.3\n')

    def test_include_unavailable(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i unavailable')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            debug=["'versions' section not found in versions.cfg.",
                   '!> http://pypi.python.org/pypi/unavailable/json 404',
                   '-> Last version of unavailable is 0.0.0.'],
            info=['- 1 packages need to be checked for updates.',
                  '> Fetching latest datas for unavailable...',
                  '- 0 package updates found.'])
        self.assertStdOut('')

    def test_include_exclude(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i unavailable -e unavailable')
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            debug=["'versions' section not found in versions.cfg."],
            info=['- 0 packages need to be checked for updates.',
                  '- 0 package updates found.'])
        self.assertStdOut('')

    def test_write_include_in_blank(self):
        config_file = NamedTemporaryFile()
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline(
                '-i egg -w %s' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[versions]\negg                             = 0.3\n')
        self.assertStdOut(
            '[versions]\negg                             = 0.3\n')

    def test_write_in_existing_file_with_exclude(self):
        config_file = NamedTemporaryFile()
        config_file.write(
            '[buildout]\ndevelop=.\n'
            '[versions]\nexcluded=1.0\negg=0.1'.encode('utf-8'))
        config_file.seek(0)
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline(
                '-e excluded -w %s' % config_file.name)
        self.assertEqual(context.exception.code, 0)
        self.assertLogs(
            debug=['-> Last version of egg is 0.3.',
                   '=> egg current version (0.1) and '
                   'last version (0.3) are different.'],
            info=['- 2 versions found in %s.' % config_file.name,
                  '- 1 packages need to be checked for updates.',
                  '> Fetching latest datas for egg...',
                  '- 1 package updates found.',
                  '- %s updated.' % config_file.name],
            warning=['[versions]',
                     'egg                             = 0.3'])
        config_file.seek(0)
        self.assertEquals(
            config_file.read().decode('utf-8'),
            '[buildout]\n'
            'develop                         = .\n\n'
            '[versions]\n'
            'excluded                        = 1.0\n'
            'egg                             = 0.3\n')
        self.assertStdOut(
            '[versions]\n'
            'egg                             = 0.3\n')

    def test_output_default(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut(
            '[versions]\n'
            'egg                             = 0.3\n')

    def test_output_with_plus_and_minus(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -vvv -qqq')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut(
            '[versions]\n'
            'egg                             = 0.3\n')

    def test_output_none(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -q')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut('')
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -qqqqqqqq')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut('')

    def test_output_increased(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -v')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut(
            '- 1 packages need to be checked for updates.\n'
            '> Fetching latest datas for egg...\n'
            '- 1 package updates found.\n'
            '[versions]\n'
            'egg                             = 0.3\n')

    def test_output_max(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -vvvvvvvvvv')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut(
            "'versions' section not found in versions.cfg.\n"
            "- 1 packages need to be checked for updates.\n"
            "> Fetching latest datas for egg...\n"
            "-> Last version of egg is 0.3.\n"
            "=> egg current version (0.0.0) and "
            "last version (0.3) are different.\n"
            "- 1 package updates found.\n"
            "[versions]\n"
            "egg                             = 0.3\n")

    def test_output_max_specifiers(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -s egg:<0.3 -vvv')
        self.assertEqual(context.exception.code, 0)
        self.assertStdOut(
            "'versions' section not found in versions.cfg.\n"
            "- 1 packages need to be checked for updates.\n"
            "> Fetching latest datas for egg...\n"
            "-> Last version of egg<0.3 is 0.2.\n"
            "=> egg current version (0.0.0) and "
            "last version (0.2) are different.\n"
            "- 1 package updates found.\n"
            "[versions]\n"
            "egg                             = 0.2\n")

    def test_specifiers_errors(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -s egg<0.3')
        self.assertEqual(context.exception.code, 2)
        self.assertInStdOut('error: argument -s/--specifier: '
                            'key:value syntax not followed')

        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i egg -s egg:')
        self.assertEqual(context.exception.code, 2)
        self.assertInStdOut('error: argument -s/--specifier: '
                            'key or value are empty')

    def test_handle_error(self):
        with self.assertRaises(SystemExit) as context:
            check_buildout_updates.cmdline('-i error-egg')
        self.assertEqual(context.exception.code,
                         "list indices must be integers, not str")


loader = TestLoader()

test_suite = TestSuite(
    [loader.loadTestsFromTestCase(VersionsCheckerTestCase),
     loader.loadTestsFromTestCase(UnusedVersionsCheckerTestCase),
     loader.loadTestsFromTestCase(VersionsConfigParserTestCase),
     loader.loadTestsFromTestCase(IndentCommandLineTestCase),
     loader.loadTestsFromTestCase(FindUnusedVersionsTestCase),
     loader.loadTestsFromTestCase(CheckUpdatesCommandLineTestCase)
     ]
)



"""Version checker for Buildout Versions Checker"""
import futures

import os
import json
import socket

from collections import OrderedDict
try:
    from urllib2 import urlopen
    from urllib2 import URLError
    from ConfigParser import NoSectionError
except ImportError:  # Python 3
    from urllib.error import URLError
    from urllib.request import urlopen
    from configparser import NoSectionError

from packaging.specifiers import SpecifierSet
from packaging.version import parse as parse_version

from bvc.logger import logger
from bvc.configparser import VersionsConfigParser


class VersionsChecker(object):
    """
    Checks updates of packages from a config file on Pypi.
    """
    default_version = '0.0.0'

    def __init__(self, source,
                 specifiers={}, allow_pre_releases=False,
                 includes=[], excludes=[],
                 service_url='http://pypi.python.org/pypi',
                 timeout=10, threads=10):
        """
        Parses a config file containing pinned versions
        of eggs and check available updates.
        """
        self.source = source
        self.includes = includes
        self.excludes = excludes
        self.specifiers = specifiers
        self.allow_pre_releases = allow_pre_releases
        self.timeout = timeout
        self.threads = threads
        self.service_url = service_url

        self.source_versions = OrderedDict(
            self.parse_versions(self.source))
        self.versions = self.include_exclude_versions(
            self.source_versions, self.includes, self.excludes)
        self.package_specifiers = self.build_specifiers(
            self.versions.keys(), self.specifiers)
        self.last_versions = OrderedDict(
            self.fetch_last_versions(self.package_specifiers,
                                     self.allow_pre_releases,
                                     self.service_url,
                                     self.timeout,
                                     self.threads))
        self.updates = OrderedDict(self.find_updates(
            self.versions, self.last_versions))

    def parse_versions(self, source):
        """
        Parses the source file to return the packages
        with their current versions.
        """
        config = VersionsConfigParser()
        config.read(source)
        try:
            versions = config.items('versions')
        except NoSectionError:
            logger.debug("'versions' section not found in %s." % source)
            return []
        logger.info('- %d versions found in %s.' % (len(versions), source))
        return versions

    def include_exclude_versions(self, source_versions,
                                 includes=[], excludes=[]):
        """
        Includes and excludes packages to be checked in
        the default dict of packages with versions.
        """
        versions = source_versions.copy()
        packages_lower = [x.lower() for x in versions.keys()]
        for include in includes:
            if include.lower() not in packages_lower:
                versions[include] = self.default_version
        excludes_lower = [x.lower() for x in excludes]
        for package in versions.keys():
            if package.lower() in excludes_lower:
                del versions[package]
        logger.info('- %d packages need to be checked for updates.' %
                    len(versions))
        return versions

    def build_specifiers(self, packages, source_specifiers):
        """
        Builds a list of tuple (package, version specifier)
        """
        specifiers = []
        source_specifiers = dict((k.lower(), v) for k, v in
                                 source_specifiers.items())
        for package in packages:
            specifier = source_specifiers.get(package.lower(), '')
            specifiers.append((package, specifier))
        return specifiers

    def fetch_last_versions(self, packages, allow_pre_releases,
                            service_url, timeout, threads):
        """
        Fetch the latest versions of a list of packages with specifiers,
        in a threaded manner or not.
        """
        versions = []
        if threads > 1:
            with futures.ThreadPoolExecutor(
                    max_workers=threads) as executor:
                tasks = [executor.submit(self.fetch_last_version,
                                         package, allow_pre_releases,
                                         service_url, timeout)
                         for package in packages]
                for task in futures.as_completed(tasks):
                    versions.append(task.result())
        else:
            for package in packages:
                versions.append(self.fetch_last_version(
                    package, allow_pre_releases, service_url, timeout))
        return versions

    def fetch_last_version(self, package, allow_pre_releases,
                           service_url, timeout):
        """
        Fetch the last version of a package on Pypi.
        """
        package, specifier = package
        specifier = SpecifierSet(specifier, allow_pre_releases)
        max_version = parse_version(self.default_version)
        logger.info('> Fetching latest datas for %s...' % package)
        package_json_url = '%s/%s/json' % (service_url, package)
        socket.setdefaulttimeout(timeout)
        try:
            content = urlopen(package_json_url).read()
        except URLError as error:
            content = '{"releases": []}'
            logger.debug('!> %s %s' % (package_json_url, error.reason))
        results = json.loads(content)
        socket.setdefaulttimeout(None)
        for version in specifier.filter(results['releases']):
            version = parse_version(version)
            if version > max_version:
                max_version = version
        logger.debug('-> Last version of %s%s is %s.' % (
            package, specifier, max_version))
        return (package, str(max_version))

    def find_updates(self, versions, last_versions):
        """
        Compare the current versions of the packages
        with the last versions to find updates.
        """
        updates = []
        for package, current_version in versions.items():
            last_version = last_versions[package]
            if last_version != current_version:
                logger.debug(
                    '=> %s current version (%s) and last '
                    'version (%s) are different.' %
                    (package, current_version, last_version))
                updates.append((package, last_version))
        logger.info('- %d package updates found.' % len(updates))
        return updates


class UnusedVersionsChecker(VersionsChecker):
    """
    Checks unused eggs in a config file.
    """

    def __init__(self, source, egg_directory, excludes=[]):
        """
        Parses a config file containing pinned versions
        of eggs and check their installation in the egg_directory.
        """
        self.source = source
        self.excludes = excludes
        self.egg_directory = egg_directory
        self.source_versions = OrderedDict(
            self.parse_versions(self.source))
        self.versions = self.include_exclude_versions(
            self.source_versions, excludes=self.excludes)
        self.used_versions = self.get_used_versions(self.egg_directory)
        self.unused = self.find_unused_versions(
            self.versions.keys(), self.used_versions)

    def get_used_versions(self, egg_directory):
        """
        Walk into the egg_directory to know the packages installed.
        """
        return [egg.split('-')[0] for egg in os.listdir(egg_directory)
                if egg.endswith('.egg')]

    def find_unused_versions(self, versions, used_versions):
        """
        Make the difference between the listed versions and
        the used versions.
        """
        unused = list(versions)
        used_version_lower = [x.lower() for x in used_versions]
        for version in versions:
            if version.lower().replace('-', '_') in used_version_lower:
                unused.remove(version)
        return unused



"""Command line for (re)indenting buildout files"""
from six import string_types

import sys
import logging
from argparse import ArgumentParser

from bvc.logger import logger
from bvc.configparser import VersionsConfigParser


def cmdline(argv=sys.argv[1:]):
    parser = ArgumentParser(
        description='(Re)indent buildout related files')
    parser.add_argument(
        'sources', nargs='*',
        help='The buildout files to (re)indent')
    format_group = parser.add_argument_group('Formatting')
    format_group.add_argument(
        '--indent', dest='indentation', type=int, default=32,
        help='Spaces used when indenting "key = value" (default: 32)')
    format_group.add_argument(
        '--sorting', dest='sorting', default='', choices=['alpha', 'length'],
        help='Sorting algorithm used on the keys when writing source file '
        '(default: None)')
    verbosity_group = parser.add_argument_group('Verbosity')
    verbosity_group.add_argument(
        '-v', action='count', dest='verbosity', default=1,
        help='Increase verbosity (specify multiple times for more)')
    verbosity_group.add_argument(
        '-q', action='count', dest='quietly', default=0,
        help='Decrease verbosity (specify multiple times for more)')

    if isinstance(argv, string_types):
        argv = argv.split()
    options = parser.parse_args(argv)

    verbose_logs = {0: 100,
                    1: logging.WARNING,
                    2: logging.INFO,
                    3: logging.DEBUG}
    verbosity = min(3, max(0, options.verbosity - options.quietly))
    console = logging.StreamHandler(sys.stdout)
    console.setLevel(verbose_logs[verbosity])
    logger.addHandler(console)

    if not options.sources:
        logger.warning('No files to (re)indent')
        sys.exit(0)

    for source in options.sources:
        config = VersionsConfigParser()
        config_readed = config.read(source)
        if config_readed:
            config.write(source, options.indentation, options.sorting)
            logger.warning('- %s (re)indented at %s spaces.' % (
                source, options.indentation))
        else:
            logger.warning('- %s cannot be read.' % source)

    sys.exit(0)



"""Setup script for bvc"""
import os

from setuptools import setup
from setuptools import find_packages

__version__ = '1.9'
__license__ = 'BSD License'

__author__ = 'Fantomas42'
__email__ = 'fantomas42@gmail.com'

__url__ = 'https://github.com/Fantomas42/buildout-versions-checker'


setup(
    name='buildout-versions-checker',
    version=__version__,
    zip_safe=False,

    packages=find_packages(exclude=['tests']),
    include_package_data=True,

    author=__author__,
    author_email=__email__,
    url=__url__,

    license=__license__,
    platforms='any',
    description='Checks egg updates in your Buildout configurations.',
    long_description=open(os.path.join('README.rst')).read(),
    keywords='buildout, versions, updates',
    classifiers=[
        'Framework :: Buildout',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    install_requires=['six',
                      'futures',
                      'packaging'],
    entry_points={
        'console_scripts': [
            'indent-buildout=bvc.scripts.indent_buildout:cmdline',
            'find-unused-versions=bvc.scripts.find_unused_versions:cmdline',
            'check-buildout-updates=bvc.scripts.check_buildout_updates:cmdline'
        ]
    },
    test_suite='bvc.tests.test_suite',
)
